#ifndef __mobile_h
#define __mobile_h

#include <string>
#include <vector>
#include <iostream>
#include "selling.h"

//------------------------------------------------------------

class Mobile {
  private:
     std::string           name;
     std::vector<Selling*> sellings;
  public:
     Mobile (const std::string& name);
     Mobile (const Mobile& mobile);

     virtual Mobile*   clone    () const = 0;
   
     std::string     getName    () const {return name;}
   
     virtual double  getDiscount() const = 0;
     virtual double  getPrice   () const = 0;
  
     virtual void print (const std::string& margin = " ") const;
   
     double   getProfit       () const;
     double   getExpense      () const;
     double   getIncome       () const;
     void     addSelling      (Selling* selling);
     int      getSellingCount () const {return sellings.size();}
     Selling* getSelling      (int i)  {return sellings[i];}
};

//------------------------------------------------------------

class BlackWhiteMobile : public Mobile {
  private:
     double discount; 
     double price;
  public:
     BlackWhiteMobile (const std::string& name, double discount, double price);
     BlackWhiteMobile (const BlackWhiteMobile& mobile);

     virtual Mobile*   clone    () const;
     virtual double  getDiscount() const {return discount;}
     virtual double  getPrice   () const {return price;}
};

//------------------------------------------------------------

class ColorMobile : public Mobile {
  private:
     double discount; 
     double price;
  public:
     ColorMobile (const std::string& name, double discount, double price);
     ColorMobile (const ColorMobile& mobile);

     virtual Mobile*   clone    () const;
     virtual double  getDiscount() const {return discount;}
     virtual double  getPrice   () const {return price;}
};

//------------------------------------------------------------

class FotoMobile : public Mobile {
  private:
     double discount; 
     double price;
  public:
     FotoMobile (const std::string& name, double discount, double price);
     FotoMobile (const FotoMobile& mobile);

     virtual Mobile*   clone    () const;
     virtual double  getDiscount() const {return discount;}
     virtual double  getPrice   () const {return price;}
};

//------------------------------------------------------------
#endif 
